"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSimpleContext = exports.SimpleContext = void 0;
const async_hooks_1 = __importDefault(require("async_hooks"));
class SimpleContext {
    constructor() {
        this.properties = {};
        this.store = new Map();
        this.asyncHooks = async_hooks_1.default.createHook({
            init: (asyncId, _, triggerAsyncId) => {
                if (this.store.has(triggerAsyncId)) {
                    this.store.set(asyncId, this.store.get(triggerAsyncId));
                }
            },
            destroy: (asyncId) => {
                if (this.store.has(asyncId)) {
                    this.store.delete(asyncId);
                }
            },
        });
        this.asyncHooks.enable();
    }
    get(key) {
        return this.getForkProperty(key);
    }
    set(key, value) {
        return this.setForkProperty(key, value);
    }
    fork() {
        this.store.set(async_hooks_1.default.executionAsyncId(), {});
        return this;
    }
    setForkProperty(key, value) {
        const store = this.store.get(async_hooks_1.default.executionAsyncId());
        store ? (store[key] = value) : (this.properties[key] = value);
    }
    getForkProperty(key) {
        const store = this.store.get(async_hooks_1.default.executionAsyncId());
        return store
            ? store[key]
            : this.properties[key];
    }
}
exports.SimpleContext = SimpleContext;
function createSimpleContext() {
    return new SimpleContext();
}
exports.createSimpleContext = createSimpleContext;
